/*
   MODULE NAME        CICSTYPE.H

   DESCRIPTIVE NAME   CICS C Language Data Types

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1994, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. The
   Gateway is available for a number of different operating
   environments and to ensure the correct sections of this header are
   included one of the following constants must be defined prior to
   inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_MVS  -  if building an OS/390 application
     CICS_LNX  -  if building a Linux application
     CICS_HPUX -  if building an HPUX application
*/

#ifndef _CICSTYPE_H
#define _CICSTYPE_H

/*
   Function linkage and pointer attributes

   CICSCALL - Linkage for CICS function entry points
   CICSEXIT - Linkage for CICS call-back functions
   CICSPTR  - Attributes for CICS data pointers
*/
#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
  #define CICSEXIT __cdecl
  #define CICSPTR
#elif defined(CICS_AIX)
  #define CICSCALL
  #define CICSEXIT
  #define CICSPTR
#elif defined(CICS_SOL)
  #define CICSCALL
  #define CICSEXIT
  #define CICSPTR
#elif defined(CICS_MVS)
  #define CICSCALL
  #define CICSEXIT
  #define CICSPTR
#elif defined(CICS_LNX)
  #define CICSCALL
  #define CICSEXIT
  #define CICSPTR
#elif defined(CICS_HPUX)
  #define CICSCALL
  #define CICSEXIT
  #define CICSPTR
#else
  #error A CICS_xxx system symbol is not defined
#endif

#if defined(CICS_HPUX)
  #pragma pack 1
#endif

/*
   Standard CICS data types

   These are the same in all Client environments.
*/
typedef char            cics_char_t;        /* Characters             */
typedef signed char     cics_sbyte_t;       /* Signed 8-bit           */
typedef unsigned char   cics_ubyte_t;       /* Unsigned 8-bit         */
typedef signed short    cics_sshort_t;      /* Signed 16-bit          */
typedef unsigned short  cics_ushort_t;      /* Unsigned 16-bit        */
typedef signed int      cics_sint_t;        /* Signed 32-bit          */
typedef unsigned int    cics_uint_t;        /* Unsigned 32-bit        */
typedef signed long     cics_slong_t;       /* Signed long            */
typedef unsigned long   cics_ulong_t;       /* Unsigned long          */
typedef void CICSPTR   *cics_ptr_t;         /* Arbitrary pointer      */
typedef int             cics_bool_t;        /* Boolean type           */
typedef unsigned short  cics_shandle_t;     /* 16-bit handle          */
typedef void CICSPTR   *cics_lhandle_t;     /* 32-bit handle          */

#if defined(CICS_HPUX) && defined(__cplusplus)
  #pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
  #pragma pack
#elif defined(CICS_HPUX)
  #pragma HP_ALIGN HPUX_NATURAL
#endif

#endif /* _CICSTYPE_H */
