/*
   MODULE NAME        DFHBMSCS.H

   DESCRIPTIVE NAME   CICS 3270 datastream attribute values

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1994, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway.
   The values it contains refer to ASCII definition of 3270 datastream
*/

#ifndef _DFHBMSCA_H
#define _DFHBMSCA_H

/* Standard attribute combination */

#define  DFHBMPEM   0x19       /* Printer end-of-message    */
#define  DFHBMPNL   '\n'       /* Printer new line          */
#define  DFHBMPFF   '\x0c'     /* Printer form-feed         */
#define  DFHBMPCR   '\x0d'     /* Printer carriage-retn     */
#define  DFHBMASK   '0'        /* Autoskip                  */
#define  DFHBMUNP   ' '        /* Unprotected               */
#define  DFHBMUNN   '&'        /* Unprotected, numeric      */
#define  DFHBMPRO   '-'        /* Protected                 */
#define  DFHBMBRY   'H'        /* High Intensity            */
#define  DFHBMDAR   '<'        /* Zero intensity, non-print */
#define  DFHBMFSE   'A'        /* MDT set                   */
#define  DFHBMPRF   '/'        /* Protected, MDT set        */
#define  DFHBMASF   '1'        /* Autoskip, MDT set         */
#define  DFHBMASB   '8'        /* Autoskip, High intensity  */
#define  DFHBMPSO   0x1E       /* DBCS - shift out          */
#define  DFHBMPSI   0x1F       /* DBCS - shift in           */

/* Field flag byte settings from input mapping */

#define  DFHBMEOF   0x80       /* Field erased              */

/* Extended attribute type codes */

#define  DFHCOLOR   0x42       /* Colour attribute code     */
#define  DFHPS      0x43       /* Programmed symbols        */
#define  DFHHLT     0x41       /* Highlight attribute code  */
#define  DFH3270    0xC0       /* 3270 field attribute code */
#define  DFHVAL     0xC1       /* Validation attribute code */
#define  DFHOUTLN   0xC2       /* Field outlining           */
                               /* attribute code            */
#define  DFHALL     0x00       /* All character attributes  */
                               /* Reset to default only     */

/* Default attribute value for setting attributes in maps */

#define  DFHDFT     0xFF       /* Default value             */

/* Colour attribute values */

#define  DFHDFCOL   0x00       /* Default colour            */
#define  DFHBLUE    '1'        /* Blue                      */
#define  DFHRED     '2'        /* Red                       */
#define  DFHPINK    '3'        /* Pink                      */
#define  DFHGREEN   '4'        /* Green                     */
#define  DFHTURQ    '5'        /* Turquoise                 */
#define  DFHYELLO   '6'        /* Yellow                    */
#define  DFHNEUTR   '7'        /* Neutral                   */

/* Base PS attribute code */

#define  DFHBASE    0x00       /* Base PS                   */
#define  DFHDFHI    0x00       /* Default highlight         */
#define  DFHBLINK   '1'        /* Blink                     */
#define  DFHREVRS   '2'        /* Reverse video             */
#define  DFHUNDLN   '4'        /* Underscore                */

/* Additional attributes */

#define  DFHUNNOD   '('        /* Unprotected               */
                               /* Non-display               */
                               /* Non-print                 */
                               /* Non-detectable            */
                               /* MDT                       */

#define  DFHUNIMD   'I'        /* Unprotected               */
                               /* Intensify                 */
                               /* MDT                       */

#define  DFHUNNUM   'J'        /* Unprotected               */
                               /* Numeric                   */
                               /* MDT                       */

#define  DFHUNINT   'R'        /* Unprotected               */
                               /* Numeric                   */
                               /* Intensify                 */
                               /* MDT                       */

#define  DFHUNNON   ')'        /* Unprotected               */
                               /* Numeric                   */
                               /* Non-display               */
                               /* Non-print                 */
                               /* Non-detectable            */
                               /* MDT                       */

#define  DFHPROTI   'Y'        /* Protected                 */
#define  DFHPROTN   '%'        /* Non-display               */
                               /* Non-print                 */
#define  DFHDFFR    0x00       /* Default outline           */
#define  DFHUNDER   0x01       /* Underline                 */
#define  DFHRIGHT   0x02       /* Right vertical line       */
#define  DFHOVER    0x04       /* Overline                  */
#define  DFHLEFT    0x08       /* Left vertical line        */
#define  DFHBOX     0x0F       /* Underline and             */
                               /* right vertical and        */
                               /* overline and              */
                               /* left vertical             */
#define  DFHSOSI    0x01       /* SOSI                      */
#define  DFHSA      0x1F       /* Set attribute order       */

#endif /* _DFHBMSCA_H */
