/* File Name(s)  : ecii1.c

   Product       : CICS Transaction Gateway

   Description   : This sample shows the basic use of the CICS Client API.
                   It queries the Client daemon for a list of available 
                   servers and then launches a transaction on the server 
                   chosen.

   Prerequisites:  This samples requires the CICS Program EC02 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


/* Include files */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <cics_eci.h>
#include "cclcalls.h"

/* Structures, defines and macro definitions */

#define ECI_ASYNC_SIZE 40

#define copyright_notice copyright4cecii1
char version4cecii1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/*  Global Variables */

CclEvent Callback;


/* Function prototypes */

void Response(short Rc, cics_char_t *Abend);
void CICSEXIT CallBackProc (cics_ushort_t AIdx);


/* Main Function */

int main(){

   CICS_EciSystem_t servers[20];             /*CICS_EciListSystems Server list  */
   cics_ushort_t    serverCount   = 20;      /*Length counter for above list    */
   cics_char_t      progid[4 + 1] = "EC02";  /*Program to call                  */
   cics_char_t      chosenServer[CICS_ECI_SYSTEM_MAX +1];/*Users selected Server*/
   ECI_PARMS        EciParms;                /*ECI Parameter block              */
   char             CommArea[ECI_ASYNC_SIZE];/*COMMAREA for use in ECI calls    */


   char             UserID[9] = "sysad   ";  /*User ID padded to 8 characters   */
   char             PassWd[9] = "sysad   ";  /*Password padded to 8 characters  */

   short            rc;
   int              i, choice = 0;
   int              loop      = 0;
   int              firstTime = 1;
   char             tempstr[255];


   /* Display Banner */

   printf("CICS Transaction Gateway Intermediate ECI V1 Sample\n\n");

   printf("Servers defined in the Client daemon:\n");


   /* Ask the Client daemon for a list of servers it has defined */

   rc = CICS_EciListSystems(NULL,&serverCount,servers);
   if (rc == ECI_NO_ERROR) {

      do {

         /* Display the list and ask for user input to choose one */

         for (i=0; i<serverCount;i++) {
            printf("   %d.%s\n",i+1 , servers[i].SystemName);
         }

         printf("\nChoose server to connect to:\n");
         scanf("%s", tempstr);
         choice = atol(tempstr);


         /* Validate input and redisplay the list if invalid */

         if (choice > 0 && choice < serverCount+1) {
            strcpy(chosenServer,servers[choice-1].SystemName);
            loop=0;
         } else {
            printf("Out of range\n");
            loop=1;
         }
      }while (loop==1);


      /* Set up an event for the callback thread to use */

      if (CreateAnEvent(NULL, &Callback) != CCLTRUE) {
         printf("Failed to create Callback event - aborting\n");
         return 1;
      }


      /* Set up the ECI Parameter block
         Note: We can reuse most of the setting for subsequent ECI calls */

      memset(CommArea, '\0', ECI_ASYNC_SIZE);
      memset(&EciParms, 0, sizeof(ECI_PARMS));

      EciParms.eci_version                           = ECI_VERSION_1A;
      EciParms.eci_callback                          = (CICS_EciNotify_t)CallBackProc;
      memcpy(&EciParms.eci_program_name, progid,       4);
      memcpy(&EciParms.eci_userid,       UserID,       8);
      memcpy(&EciParms.eci_password,     PassWd,       8);
      memcpy(&EciParms.eci_system_name,  chosenServer, 8);
      EciParms.eci_commarea                          = CommArea;
      EciParms.eci_commarea_length                   = ECI_ASYNC_SIZE;
      EciParms.eci_extend_mode                       = ECI_EXTENDED;
      EciParms.eci_luw_token                         = ECI_LUW_NEW;
      EciParms.eci_timeout                           = 30;


      /* Call the chosen server with the given parameters for a NEW LUW */

      do {
         if (firstTime == 1) {
            printf("\nNew LUW started for server %s\n", chosenServer);
            firstTime = 0;
         } else {
            printf("\nContinuing LUW for server %s\n", chosenServer);
         }


         /* Set called type to asynchronous and make ECI call to server */

         EciParms.eci_call_type                       = ECI_ASYNC;

         rc = CICS_ExternalCall (&EciParms);
         if (rc == ECI_NO_ERROR) {

            /* Wait for the callback routine to be driven and the event flagged */

            if (WaitOnAnEvent( &Callback ) != CCLTRUE) {
               printf("Failed to wait for callback to complete - aborting\n");
               return 1;
            }


            /* Reset the event so it can be reused */

            if (ResetAnEvent( &Callback ) != CCLTRUE) {
               printf("Cannot reset the event to be reused - aborting\n");
               return 1;
            }


            /* Now get the reply by setting the call type to ECI_GET_REPLY
               and making another CICS_ExternalCall() */

            EciParms.eci_call_type                 = ECI_GET_REPLY;
            rc = CICS_ExternalCall (&EciParms);
            if (rc == ECI_NO_ERROR) {
               printf("Program %s returned with data: %s\n", progid, CommArea);
            } else {
               Response(rc, EciParms.eci_abend_code);
               return 1;
            }
         } else {
            Response(rc, EciParms.eci_abend_code);
            return 1;
         }


         /*Ask user whether they want to run another transaction for the LUW */

         printf("\n\nDo you wish to run the program again in this LUW");
         printf(" (0=no, 1=yes)?\n");

         scanf("%s", tempstr);
         choice = atol(tempstr);

      } while (choice == 1);


      /* Now decide whether we should commit or backout the changes */

      printf("\n\nDo you wish to commit the changes for this LUW");
      printf(" (0=BACKOUT, 1=COMMIT)?\n");

      scanf("%s", tempstr);
      choice = atol(tempstr);


      /* Set up the ECI Parm block to either commit or backout appropriately */

      EciParms.eci_call_type                        = ECI_SYNC;
      memset(&EciParms.eci_program_name, '\0', 4);
      if (choice == 1) {
         EciParms.eci_extend_mode                   = ECI_COMMIT;
      } else {
         EciParms.eci_extend_mode                   = ECI_BACKOUT;
      }


      /* Make the call to either commit or backout the LUW */

      rc = CICS_ExternalCall (&EciParms);
      if (rc == ECI_NO_ERROR) {
         printf("Finished with LUW for server %s\n", chosenServer);
      } else {
         Response(rc, EciParms.eci_abend_code);
         return 1;
      }


      /* Tidy up the event which was created */

      DeleteAnEvent(&Callback);

   } else {
      Response(rc, NULL);
      return 1;
   }

   return 0;
}


/* Function: CallBackProc

   Purpose: Set the flag if the callback is ready */

void CICSEXIT CallBackProc (cics_ushort_t AIdx)
{
   if (FlagAnEvent( &Callback ) != CCLTRUE) {
      printf("Failed to post the callback flag - aborting\n");
   }
   return;
}


/* Function: Response

   Purpose:  Display the immediate response code from an ECI call */

void Response(short Rc, cics_char_t *Abend)
{
   char *p;
   char *abendString;

   switch (Rc) {
   case ECI_NO_ERROR:                p = "ECI_NO_ERROR               ";break;
   case ECI_ERR_INVALID_DATA_LENGTH: p = "ECI_ERR_INVALID_DATA_LENGTH";break;
   case ECI_ERR_INVALID_EXTEND_MODE: p = "ECI_ERR_INVALID_EXTEND_MODE";break;
   case ECI_ERR_NO_CICS:             p = "ECI_ERR_NO_CICS            ";break;
   case ECI_ERR_CICS_DIED:           p = "ECI_ERR_CICS_DIED          ";break;
   case ECI_ERR_REQUEST_TIMEOUT:     p = "ECI_ERR_REQUEST_TIMEOUT    ";break;
   case ECI_ERR_RESPONSE_TIMEOUT:    p = "ECI_ERR_RESPONSE_TIMEOUT   ";break;
   case ECI_ERR_TRANSACTION_ABEND:   p = "ECI_ERR_TRANSACTION_ABEND  ";break;
   case ECI_ERR_LUW_TOKEN:           p = "ECI_ERR_LUW_TOKEN          ";break;
   case ECI_ERR_SYSTEM_ERROR:        p = "ECI_ERR_SYSTEM_ERROR       ";break;
   case ECI_ERR_NULL_WIN_HANDLE:     p = "ECI_ERR_NULL_WIN_HANDLE    ";break;
   case ECI_ERR_NULL_MESSAGE_ID:     p = "ECI_ERR_NULL_MESSAGE_ID    ";break;
   case ECI_ERR_THREAD_CREATE_ERROR: p = "ECI_ERR_THREAD_CREATE_ERROR";break;
   case ECI_ERR_INVALID_CALL_TYPE:   p = "ECI_ERR_INVALID_CALL_TYPE  ";break;
   case ECI_ERR_ALREADY_ACTIVE:      p = "ECI_ERR_ALREADY_ACTIVE     ";break;
   case ECI_ERR_RESOURCE_SHORTAGE:   p = "ECI_ERR_RESOURCE_SHORTAGE  ";break;
   case ECI_ERR_NO_SESSIONS:         p = "ECI_ERR_NO_SESSIONS        ";break;
   case ECI_ERR_NULL_SEM_HANDLE:     p = "ECI_ERR_NULL_SEM_HANDLE    ";break;
   case ECI_ERR_INVALID_DATA_AREA:   p = "ECI_ERR_INVALID_DATA_AREA  ";break;
   case ECI_ERR_INVALID_VERSION:     p = "ECI_ERR_INVALID_VERSION    ";break;
   case ECI_ERR_UNKNOWN_SERVER:      p = "ECI_ERR_UNKNOWN_SERVER     ";break;
   case ECI_ERR_CALL_FROM_CALLBACK:  p = "ECI_ERR_CALL_FROM_CALLBACK ";break;
   case ECI_ERR_MORE_SYSTEMS:        p = "ECI_ERR_MORE_SYSTEMS       ";break;
   case ECI_ERR_NO_SYSTEMS:          p = "ECI_ERR_NO_SYSTEMS         ";break;
   case ECI_ERR_SECURITY_ERROR:      p = "ECI_ERR_SECURITY_ERROR     ";break;
   case ECI_ERR_MAX_SYSTEMS:         p = "ECI_ERR_MAX_SYSTEMS        ";break;
   case ECI_ERR_MAX_SESSIONS:        p = "ECI_ERR_MAX_SESSIONS       ";break;
   case ECI_ERR_ROLLEDBACK:          p = "ECI_ERR_ROLLEDBACK         ";break;
   default:
      {
         p = "!!!Unknown!!!";
         printf("Unknown return code : %d\n", Rc);
      }
      break;
   }

   printf("ECI returned: %s\n", p);
   if (Rc != ECI_NO_ERROR) {
      if (Abend==NULL) {
         abendString="NULL";
      } else {
         abendString=Abend;
      }
      printf("Abend code was %4.4s \n", abendString);
      exit(0);
   }

}
