      *
      * Module:         EPIB1.CBL
      *
      * Description:    Basic CICS TG Client API Cobol Sample using the
      *                 External Presentation Interface
      *
      * Prerequisites:  For successful operation this samples requires
      *                 the CICS transaction EP01 installed on your
      *                 CICS server. The source for EP01 is installed
      *                 as <install_path>/samples/server/ep01.ccp
      *
      *                 Before compiling with Microfocus Cobol this
      *                 sample program needs to be modified, the
      *                 instructions on what to edit are included
      *                 in the program comments.
      *
      * NOTE:
      *
      * The following code is sample code created by IBM Corporation.
      * This sample code is not part of any standard IBM product and
      * is provided to you solely for the purpose of assisting you in
      * the development of your applications.  The code is provided
      * 'AS IS', without warranty or condition of any kind.  IBM shall
      * not be liable for any damages arising out of your use of the
      * sample code, even if IBM has been advised of the possibility
      * of such damages.
      *
      *
      * Licensed Materials - Property of IBM  
      *  
      * 5724-I81,5725-B65 
      *  
      * (C) Copyright IBM Corp. 1994, 2014 All Rights Reserved.  
      *  
      * US Government Users Restricted Rights - Use, duplication or  
      * disclosure restricted by GSA ADP Schedule Contract with  
      * IBM Corp.  
      *
      * Status: Version 9 Release 1 
      *
       IDENTIFICATION DIVISION.
       PROGRAM-ID.    EPIB1.

       ENVIRONMENT DIVISION.
      * To compile for Microfocus Cobol uncomment the following
      * two lines.
      *SPECIAL-NAMES.
      *CALL-CONVENTION 8 IS CICS.

       DATA DIVISION.
       WORKING-STORAGE SECTION.

      * Variable definitions
       01 WORKING-VARIABLES.
           02 WS-DEBUG-ON-FLAG      PIC X VALUE 'N'.
              88  DEBUG-ON          VALUE 'Y'.
           02 WS-EPI-RETURN-CODE    PIC -9999.
           02 NULL-PTR              POINTER.
      * Used by EpiListSystems
           02 NAMESPACE             POINTER.
           02 SYSTEMS               PIC 9(4)  COMP-5.
           02 LOOP-COUNT            PIC 9(1)  COMP-5.
           02 WS-LOOP-COUNT         PIC 9(1).
           02 WS-SERVER             PIC X(8) VALUE LOW-VALUES.
      * Used by EpiStartTran
           02 SEND-BUFFER-PTR       POINTER.
           02 SEND-BUFFER.
               05 SEND-3270-AID     PIC X          VALUE X'27'.
               05 SEND-BUFF-BYTE0   PIC X          VALUE X'20'.
               05 SEND-BUFF-BYTE1   PIC X          VALUE X'20'.
               05 SEND-TRAN-CODE    PIC X(4)       VALUE 'EP01'.
               05 SEND-DATA         PIC X(50).
      * Used by EpiGetEvent
           02 RECV-BUFFER-PTR       POINTER.
           02 RECV-BUFFER.
               05 RECV-3270         PIC X(2).
               05 RECV-BUFF-DATA    PIC X(200).
           02 RECV-BUFF-SIZE        PIC 9(4) COMP-5 VALUE 202.
      * Used by EpiReply
           02 REPLY-BUFFER-PTR      POINTER.
           02 REPLY-BUFFER.
               05 REPLY-3270-AID    PIC X          VALUE X'27'.
               05 REPLY-BUFF-BYTE0  PIC X          VALUE X'20'.
               05 REPLY-BUFF-BYTE1  PIC X          VALUE X'20'.
               05 REPLY-INPUT       PIC X(17).

      * Used by Response
           02  RESPONSE-TEXT PIC X(30) VALUE 'CICS-EPI-NORMAL'.
           02  NAME-TEXT     PIC X(30) VALUE SPACES.
       02  WS-CICS-EPI-ERRORS.
      *    Cobol starts subscripting from 1 so return-code(0)
      *    i.e. CICS-EPI-NORMAL is not included in this array so as to
      *    keep this array in step with the error numbers in CICSEPI.CBL
      *    05  FILLER PIC X(30) VALUE 'CICS-EPI-NORMAL               '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NOT-INIT         '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-BAD-INDEX        '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-FAILED           '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-VERSION          '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-IS-INIT          '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NO-SYSTEMS       '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MORE-SYSTEMS     '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MAX-TERMS        '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SYSTEM           '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-TRAN-ACTIVE      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NO-DATA          '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-TTI-ACTIVE       '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-ATI-ACTIVE       '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NO-CONVERSE      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-ATI-STATE        '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NO-EVENT         '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MORE-EVENTS      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MORE-DATA        '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-WAIT             '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SERVER-DOWN      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SENSE-CODE       '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SECURITY         '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-IN-CALLBACK      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NULL-PARM        '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-ADDTYPE-INVALID  '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SIGNONCAP-INVALID'.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-RESPONSE-TIMEOUT '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SIGNON-NOT-POSS  '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-TERMID-INVALID   '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MODELID-INVALID  '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NOT-3270-DEVICE  '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-ALREADY-INSTALLED'.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-USERID-INVALID   '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-PASSWORD-INVALID '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NULL-USERID      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NULL-PASSWORD    '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-CCSID-INVALID    '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-SERVER-BUSY      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-ABENDED          '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-RESOURCE-SHORTAGE'.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-NO-CICS          '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MAX-SYSTEMS      '.
           05  FILLER PIC X(30) VALUE 'CICS-EPI-ERR-MAX-SESSIONS     '.

       02  WS-TBL-CICS-EPI-ERRORS REDEFINES WS-CICS-EPI-ERRORS.
           05  WS-CICS-EPI-ERROR-LINE OCCURS 43 TIMES.
               10  WS-CICS-EPI-ERROR  PIC X(30).

      * CICS EPI API variable definitions
           COPY "cicsepi.cbl".


       PROCEDURE DIVISION.

      * MAIN - Control section
       MAIN SECTION.

           DISPLAY 'Initialising EPI'.
           DISPLAY ' '.
           PERFORM A-INITIALIZE.
           PERFORM B-LIST-SYSTEMS.
           PERFORM C-ADD-TERMINAL.
           PERFORM D-START-TRANSACTION.
      *     /* Get some events until we get an END-TRAN event
      *        or  the transaction fails or GetEvent fails */
           PERFORM E-GET-EVENT UNTIL CICS-EPI-EVENT-END-TRAN
                               OR NOT (CICS-EPI-NORMAL
                                  OR CICS-EPI-ERR-MORE-EVENTS).
           IF CICS-EPI-EVENT-END-TRAN
           THEN
               DISPLAY 'End of transaction - end reason: ' EndReason
      *        /* Check EndReason */
               IF CICS-EPI-TRAN-NO-ERROR
               THEN
                   CONTINUE
               ELSE
                   MOVE 'AddTerm failure returned by GetEvent'
                     TO NAME-TEXT
                   MOVE ENDRETURNCODE TO EPI-RETURN-CODE
                   PERFORM XA-RESPONSE
               END-IF
           END-IF.
           PERFORM F-DEL-TERMINAL.
      *     /* Get some events until we get an END-TERM event */
           INITIALIZE CICS-EPIEVENTDATA.
           PERFORM E-GET-EVENT UNTIL CICS-EPI-EVENT-END-TERM
                               OR NOT (CICS-EPI-NORMAL
                                  OR CICS-EPI-ERR-MORE-EVENTS).
           IF CICS-EPI-EVENT-END-TERM
           THEN
               DISPLAY 'Terminal deleted - end reason: ' EndReason
      *        /* Check EndReason */
               IF CICS-EPI-TRAN-NO-ERROR
               THEN
                   CONTINUE
               ELSE
                   MOVE 'DelTerm failure returned by GetEvent'
                     TO NAME-TEXT
                   MOVE ENDRETURNCODE TO EPI-RETURN-CODE
                   PERFORM XA-RESPONSE
               END-IF
           END-IF.
           PERFORM G-EPI-TERMINATE.

           DISPLAY  ' '
           DISPLAY  'EPI Sample Program Completed'

           PERFORM XE-EXIT-PROG.

           EXIT.


      * Issue a CICS_EpiInitialize call
       A-INITIALIZE SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiInitialize called' TO NAME-TEXT
           END-IF.
           SET NULL-PTR TO NULL.
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPIINITIALIZE'
           CALL 'CICSEPIINITIALIZE'

                USING BY VALUE CICS-EPI-VERSION-200
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE

           EXIT.


      * Issue a CICS_EpiListSystems call
       B-LIST-SYSTEMS SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiListSystems called' TO NAME-TEXT
           END-IF.
           SET NAMESPACE TO NULL.
           MOVE 1 TO CICS-EPINUMSYS.
           MOVE 9 TO SYSTEMS.

      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPILISTSYSTEMS'
           CALL 'CICSEPILISTSYSTEMS'
                USING BY REFERENCE NAMESPACE
                      BY REFERENCE SYSTEMS
                      BY REFERENCE CICS-EPISYSTEM
                RETURNING EPI-RETURN-CODE.

           IF CICS-EPI-NORMAL OR CICS-EPI-ERR-MORE-SYSTEMS
                 PERFORM WITH TEST BEFORE
                         VARYING LOOP-COUNT FROM 1 BY 1
                         UNTIL LOOP-COUNT > SYSTEMS
                            OR LOOP-COUNT > 9
                            OR SYSTEMNAME(LOOP-COUNT) <= SPACES
                     MOVE LOOP-COUNT TO WS-LOOP-COUNT
                     DISPLAY WS-LOOP-COUNT
                             '  '
                             SYSTEMNAME(LOOP-COUNT)
                             '       '
                             SYSTEMDESC(LOOP-COUNT)
                 END-PERFORM
           END-IF

           PERFORM XA-RESPONSE

           DISPLAY 'Choose Server to connect to: '
           ACCEPT WS-LOOP-COUNT.
           MOVE WS-LOOP-COUNT TO LOOP-COUNT.
           MOVE SYSTEMNAME(LOOP-COUNT) TO WS-SERVER.
           DISPLAY 'Creating a Terminal on Server: ' WS-SERVER ' '.

        B-EXIT.
           EXIT.

      *
      * Issue a CICS_EpiAddTerminal call
       C-ADD-TERMINAL SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiAddTerminal called' TO NAME-TEXT
           END-IF.
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPIADDTERMINAL'
           CALL 'CICSEPIADDTERMINAL'
                USING BY VALUE     NULL-PTR
                      BY REFERENCE WS-SERVER
                      BY VALUE     NULL-PTR
                      BY VALUE     NULL-PTR
                      BY VALUE     NULL-PTR
                      BY REFERENCE CICS-EPIDETAILS
                      BY REFERENCE TERMINDEX
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE.

           IF NOT (CICS-EPI-NORMAL OR CICS-EPI-ERR-MORE-EVENTS)
              PERFORM XE-EXIT-PROG
           ELSE
              DISPLAY 'Connected on Terminal: ' TERMID
              DISPLAY 'Terminal dimensions = '  NUMLINES 'x' NUMCOLUMNS
           END-IF.

        C-EXIT.
           EXIT.

      * Issue a CICS_EpiStartTran call, to the transaction EP01,
      * passing a string of data
       D-START-TRANSACTION SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiStartTran called' TO NAME-TEXT
           END-IF.
           MOVE '        0th run of EP01'    TO SEND-DATA.
           DISPLAY 'Starting ' SEND-TRAN-CODE ' on ' WS-SERVER ' '.

      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPISTARTTRAN'
           CALL 'CICSEPISTARTTRAN'
                USING BY VALUE     TERMINDEX
                      BY VALUE     NULL-PTR
                      BY REFERENCE SEND-BUFFER
                      BY VALUE     80
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE.

        D-EXIT.
           EXIT.

      * Issue a CICS_EpiGetEvent, the reply will indicate
      * what to do next
       E-GET-EVENT SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiGetEvent called' TO NAME-TEXT
           END-IF.
           SET DATAPOINTER TO ADDRESS OF RECV-BUFFER.
           MOVE RECV-BUFF-SIZE TO DATASIZE.
           SET CICS-EPI-WAIT TO TRUE.

      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPIGETEVENT'
           CALL 'CICSEPIGETEVENT'
                USING BY VALUE     TERMINDEX
                      BY VALUE     CICS-EPI-WAIT-REQUEST
                      BY REFERENCE CICS-EPIEVENTDATA
                RETURNING EPI-RETURN-CODE.
           MOVE EPI-RETURN-CODE TO WS-EPI-RETURN-CODE
           IF (CICS-EPI-NORMAL OR CICS-EPI-ERR-MORE-EVENTS)
           THEN
             IF NOT (CICS-EPI-EVENT-END-TRAN
                     OR CICS-EPI-EVENT-END-TERM)
             THEN
               EVALUATE TRUE
               WHEN CICS-EPI-EVENT-SEND
                 PERFORM XB-EVENT-SEND
               WHEN CICS-EPI-EVENT-CONVERSE
                 PERFORM XC-EVENT-CONVERSE
               WHEN OTHER
                 IF NOT CICS-EPI-NORMAL
                     DISPLAY 'Unexpected return code from EpiGetEvent: '
                         EPI-RETURN-CODE
                 END-IF
                 PERFORM XE-EXIT-PROG
               END-EVALUATE
             END-IF
           END-IF.

        E-EXIT.
           EXIT.

      * Issue a CICS_EpiDelTerminal call
       F-DEL-TERMINAL SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiDelTerminal called' TO NAME-TEXT
           END-IF.
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPIDELTERMINAL'
           CALL 'CICSEPIDELTERMINAL'
                USING BY VALUE     TERMINDEX
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE.

        F-EXIT.
           EXIT.

      * Issue a CICS_EpiTerminate call
       G-EPI-TERMINATE SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiTerminate called' TO NAME-TEXT
           END-IF.
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPITERMINATE'
           CALL 'CICSEPITERMINATE'
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE.

        G-EXIT.
           EXIT.

      * Display the immediate response code from an EPI call
       XA-RESPONSE SECTION.

           IF DEBUG-ON
               DISPLAY NAME-TEXT
           END-IF.

           MOVE EPI-RETURN-CODE        TO WS-EPI-RETURN-CODE.
           EVALUATE EPI-RETURN-CODE
           WHEN     ZERO
               MOVE 'CICS-EPI-NORMAL'                  TO RESPONSE-TEXT
           WHEN     1 THRU 43
               MOVE WS-CICS-EPI-ERROR(EPI-RETURN-CODE) TO RESPONSE-TEXT
           WHEN OTHER
               STRING 'CICS-EPI-ERR-OTHER !!!Unknown!!!'
                      WS-EPI-RETURN-CODE
                   DELIMITED BY SIZE INTO RESPONSE-TEXT
           END-EVALUATE.

           IF  NOT (CICS-EPI-NORMAL
                OR  CICS-EPI-ERR-MORE-SYSTEMS
                OR  CICS-EPI-ERR-MORE-EVENTS)
              PERFORM XD-ERROR
           END-IF.

        XA-EXIT.
           EXIT.


      * Some data has been sent, so display it
       XB-EVENT-SEND SECTION.

           IF RECV-BUFF-SIZE GREATER THAN ZERO
           THEN
             DISPLAY 'The following message was received from CICS'
             DISPLAY ' '
             DISPLAY RECV-BUFF-DATA
           END-IF.

        XB-EXIT.
           EXIT.


      * Issue a CICS_EpiReply call, to the transaction EP01,
      * with a string
       XC-EVENT-CONVERSE SECTION.

           IF DEBUG-ON
               MOVE 'CICS_EpiReply done' TO NAME-TEXT
           END-IF.

           MOVE 'This is my reply:' TO REPLY-INPUT.

      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEPIREPLY'
           CALL 'CICSEPIREPLY'
                USING BY VALUE     TERMINDEX
                      BY REFERENCE REPLY-BUFFER
                      BY VALUE     20
                RETURNING EPI-RETURN-CODE.

           PERFORM XA-RESPONSE.

        XC-EXIT.
           EXIT.


      * Display the System Error response from an EPI Call
       XD-ERROR SECTION.

           DISPLAY  'System Error:' EPI-RETURN-CODE
                    'Cause:'    RESPONSE-TEXT.

           PERFORM XE-EXIT-PROG.

       XD-EXIT.
           EXIT.


      * End program
       XE-EXIT-PROG SECTION.

           GOBACK.

       XE-EXIT.
           EXIT.
      *End of program
