/*               CICS TG Client API Classes

   FILE NAME:    cclbuf.hpp

   CLASS:        CclBuf

   DESCRIPTION:  Class to simplify manipulation of data areas

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 
*/


#ifndef _CCLBUF_
#define _CCLBUF_

class CclBuf {
public:


// enumerations

  enum           DataAreaType  { fixed,extensible };
  enum           DataAreaOwner { internal,external };


// constructors/destructors

  CCLI           CclBuf(                     //allocation constructor..internal
                   unsigned long length = 0,         //initial data area length
                   DataAreaType type = extensible );     //fixed or extensible?
  CCLI           CclBuf(                  //overlay constructor..fixed,external
                   unsigned long length,                //size of external area
                   void* dataArea );            //address of external data area
  CCLI           CclBuf(                    //string-copy constructor..internal
                   const char* text,      //zero-terminated string to be copied
                   DataAreaType type = extensible );     //fixed or extensible?
  CCLI           CclBuf(                           //copy constructor..internal
                   const CclBuf& buffer );                //buffer to be copied
  CCLI virtual   ~CclBuf();


// operators

  CCLI CclBuf&   operator=(                        //assign from another buffer
                   const CclBuf& buffer );                      //source buffer
  CCLI CclBuf&   operator=(              //assign from a zero-terminated string
                   const char* text );                          //source string
  CCLI CclBuf&   operator+=(                       //append from another buffer
                   const CclBuf& buffer );                      //source buffer
  CCLI CclBuf&   operator+=(             //append from a zero-terminated string
                   const char* text );                          //source string
  CCLI Ccl::Bool operator==(                                //equality operator
                   const CclBuf& buffer ) const;            //buffer to compare
  CCLI Ccl::Bool operator!=(                              //inequality operator
                   const CclBuf& buffer ) const;            //buffer to compare


// actions

  CCLI CclBuf&   assign(                                //assign from data area
                   unsigned long length,             //size of source data area
                   const void* dataArea );        //address of source data area
  CCLI CclBuf&   cut(                                    //cut data from buffer
                   unsigned long length,                        //length of cut
                   unsigned long offset = 0 );             //offset into buffer
  CCLI CclBuf&   insert(                       //insert source data into buffer
                   unsigned long length,                //length of source data
                   const void* dataArea,                  //source data address
                   unsigned long offset = 0 );             //offset into buffer
  CCLI CclBuf&   replace(             //replace data in buffer with source data
                   unsigned long length,                //length of source data
                   const void* dataArea,                  //source data address
                   unsigned long offset = 0 );             //offset into buffer


// attributes

  CCLI const void*
                 dataArea(                     //address of data area in buffer
                   unsigned long offset = 0 ) const;    //offset into data area
  CCLI unsigned long
                 dataAreaLength() const;        //length of data area in buffer
  CCLI DataAreaOwner
                 dataAreaOwner() const;                 //internal or external?
  CCLI DataAreaType
                 dataAreaType() const;                   //fixed or extensible?
  CCLI unsigned long
                 dataLength() const;              //length of data in data area
  CCLI const char*
                 listState() const;                                //state text
  CCLI unsigned long
                 setDataLength(
                   unsigned long length );                    //new data length
private:
  CclBufI*       ivpI;                                                //private
};

#endif
