/*              CICS TG Client API Classes

   FILE NAME:   cclfield.hpp

   DESCRIPTION: C++ declaration of CICS TG Client API EPI Field Class
                (CclField)

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 
*/


#ifndef _CCLFIELD_
#define _CCLFIELD_

class CclField {
friend class CclScreen;
friend class CclScreenI;
public:

// Typedefs and enumerations

  enum           BaseProt {            // base attribute, protected input
                   protect, unprotect };
  enum           BaseType {            // base attribute, input data type
                   alphanumeric, numeric };
  enum           BaseInts {            // base attribute, display intensity
                   normal, intense, dark };
  enum           BaseMDT  {            // base attribute, modified data tag
                   unmodified, modified };

  enum           Color    {            // extended attribute, color
                   defaultColor, blue, red, pink, green, cyan, yellow,
                   neutral, black, darkBlue, orange, purple, paleGreen,
                   paleCyan, gray, white };
  enum           Highlight {           // extended attribute, highlighting
                   defaultHlt, normalHlt, blinkHlt, reverseHlt, underscoreHlt, intenseHlt };
  enum           Transparency {        // extended attribute, transparency
                   defaultTran, orTran, xorTran, opaqueTran };


// Constructors/Destructors

  CCLI           CclField( unsigned short position );
  CCLI           CclField( unsigned short position, CclScreen *screen );
  CCLI virtual   ~CclField();


// Accessors for Field Size/Position

  CCLI unsigned short
                 position() const;     // get field position (buffer offset)
  CCLI unsigned short
                 row() const;          // get field row
  CCLI unsigned short
                 column() const;       // get field column
  CCLI unsigned short
                 length() const;       // get field total length


// Accessors for Field Text Contents

  CCLI const char*
                 text() const;         // get field text
  CCLI unsigned short
                 textLength() const;   // get field text length
  CCLI void      setText(              // set field text (specified length)
                   const char* text,
                   unsigned short length );
  CCLI void      setText(              // set field text (null terminated)
                   const char* text );
  CCLI void      appendText(           // append field text (specified length)
                   const char* text,
                   unsigned short length );
  CCLI void      appendText(           // append field text (null terminated)
                   const char* text );


// Accessors for Field Attributes

  CCLI BaseProt  inputProt() const;    // get base attribute, input protection
  CCLI BaseType  inputType() const;    // get base atribute, input data type
  CCLI BaseInts  intensity() const;    // get base atribute, display intensity
  CCLI BaseMDT   dataTag() const;      // get base attribute, modified data tag

  CCLI char      baseAttribute() const;   // get base attribute as single byte
  CCLI void      setBaseAttribute(        // set base attribute as single byte
                   char attribute );
  CCLI void      resetDataTag();          // reset modified data tag

  CCLI Highlight highlight() const;       // get extended highlighting
  CCLI Transparency
                 transparency() const;    // get extended transparency
  CCLI Color     foregroundColor() const; // get extended foreground color
  CCLI Color     backgroundColor() const; // get extended background color

  CCLI void      setExtAttribute(         // set extended attribute pair
                   unsigned char attribute,      // attribute id
                   unsigned char value );        // attribute value

private:
  CclFieldI*     ivpI;                 // pointer to implementation class

}; // end of class CclField

#endif
