/*               CICS TG Classes

   FILE NAME:    cclflo.hpp

   CLASS:        CclFlow

   DESCRIPTION:  Class to simplify flow synchronisation

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 
*/


#ifndef _CCLFLO_
#define _CCLFLO_

class CclFlow {
friend class CclUOW;
friend class CclConn;
public:

// enumeration

  enum           CallType {inactive,link,backout,commit,status,changed,cancel};


// constructors/destructors

  CCLI           CclFlow(
                   Ccl::Sync syncType,                   //synchronisation type
                   unsigned long stackPages = 3);        //stack pages, if async
  CCLI           CclFlow(
                   Ccl::Sync syncType,                   //synchronisation type
                   unsigned long stackPages,             //stack pages, if async
                   const unsigned short &timeout);
  CCLI virtual  ~CclFlow();


// actions

  CCLI void      forceReset();                //make flow inactive and reset it
  CCLI virtual void
                 handleReply(
                   CclBuf* commarea );                //commarea buffer, if any
  CCLI Ccl::Bool poll(
                   CclBuf* commarea = 0 );            //commarea buffer, if any
  CCLI void      wait();                          //wait until flow is inactive


// attributes

  CCLI const char*
                 abendCode() const;               //most recent CICS abend code
  CCLI CallType  callType() const;                          //current call type
  CCLI const char*
                 callTypeText() const;                //current call type..text
  CCLI CclConn*  connection() const;                //server connection, if any
  CCLI const char*
                 diagnose() const;                            //diagnostic text
  CCLI unsigned short
                 flowId() const;                              //flow identifier
  CCLI const char*
                 listState() const;                                //state text
  CCLI Ccl::Sync syncType() const;                       //synchronisation type
  CCLI CclUOW*   uow() const;                           //UOW, if participating
  CCLI unsigned short
                 timeout() const;                      // Query Current Timeout
  CCLI void      setTimeout(const unsigned short&);// Set Timeout for next flow
private:
                 CclFlow( const CclFlow& );         //disallow copy constructor
  CclFlow&       operator=( const CclFlow& );             //disallow assignment
  CclFlowI*      ivpI;                                                //private
};

#endif
