/*              CICS TG Classes

   FILE NAME:   cclscrn.hpp

   DESCRIPTION: C++ declaration of CICS TG EPI Screen Class (CclScreen)

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 
*/


#ifndef _CCLSCRN_
#define _CCLSCRN_

class CclScreen {
friend class CclScreenI;
friend class CclTerminal;
friend class CclTerminalI;
public:

// Typedefs and enumerations

  enum           AID {
                   enter, clear, PA1, PA2, PA3,
                   PF1, PF2, PF3, PF4, PF5, PF6, PF7, PF8,
                   PF9, PF10 ,PF11, PF12, PF13, PF14 ,PF15, PF16,
                   PF17 ,PF18, PF19, PF20, PF21, PF22, PF23, PF24 };


// Constructors/Destructors

  CCLI           CclScreen(
                     unsigned short rows, unsigned short cols );
  CCLI virtual   ~CclScreen();


// Actions

  CCLI CclField* field(                // get field by index
                   unsigned short index );
  CCLI CclField* field(                // get field by position
                   unsigned short row,
                   unsigned short column );
  CCLI void      setAID(               // set outbound 3270 attention identifier
                   const AID key );
  CCLI void      setCursor(            // set cursor position on screen
                   unsigned short row,
                   unsigned short col );


// Accessors

  CCLI unsigned short
                 fieldCount() const;   // get number of fields on screen
  CCLI unsigned short
                 depth() const;        // get screen depth (rows)
  CCLI unsigned short
                 width() const;        // get screen width (columns)

  CCLI unsigned short
                 cursorRow() const;    // query cursor row
  CCLI unsigned short
                 cursorCol() const;    // query cursor column
  CCLI const char*
                 mapName() const;      // query the MapName
  CCLI const char*
                 mapSetName() const;   // query the MapSet Name

protected:

// Actions

  CCLI virtual CclField*
                 createField(         // override for CclField creation
                   unsigned short position );       // 3270 buffer offset

private:
  CclScreenI*    ivpI;                 // pointer to implementation class

}; // end class CclScreen

#endif
