/* File Name(s)  : epii1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample demonstrates basic use of Adding a terminal
                   using the new features available such as asynchronous
                   terminal install, install timeout, signon capability,
                   read timeout and purging a terminal. It also introduces
                   the basics of how to implement handleReply and
                   handleException.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind. IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


// Include files

#include "cicsepi.hpp"

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include "iostream.h"
#include "iomanip.h"
#endif

#include "stdlib.h"
#include "cclcalls.h"


// Function Headers

void DoOtherThings();
unsigned short GetResponse(CclScreen::AID&);


// Global variables and defines

CclEvent eventOccurred;
Ccl::Bool connected;
unsigned short eventReceived;

#define EVENT_EXCEPTION 1
#define EVENT_REPLY     2

#define RESP_QUIT       0
#define RESP_OK         1
#define RESP_BADINPUT   2

#define copyright_notice copyright4cppepii1
char version4cppepii1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";



/* Class: MyCclEPI
   Need to subclass CclEPI to provide an exception handler that is valid for
   an asynchronous environment */

class MyCclEPI : public CclEPI
{
public:
   void handleException(CclException ex) {
      switch (ex.exCode()) {
      case Ccl::signonNotPossible:
         cout << "You cannot create a signon capable terminal on AS/400."
              << endl;
         exit(0);
         break;
      case Ccl::timeout:
        if (strcmp(ex.methodName(),"install") == 0 ) {


           // Terminal Failed to install due to a timeout.

           cout << "Timeout while trying install the terminal. Exiting." << endl;
           exit(0);
        }
        else {


           // A read timeout occurred, the transaction will have ended

           cout << "A timeout occurred. Transaction has ended." << endl;
           cout << "This program is not designed to be able to continue, so must end."
                << endl;
           eventReceived = EVENT_EXCEPTION;
           FlagAnEvent(&eventOccurred);
        }
        break;
      default:


         // An Unhandled Exception Occurred. Display and kill the program.

         cout << "An unhandled CICS exception occurred." << endl;
         cout << ex.diagnose() << endl;
         cout << "The program cannot continue." << endl;
         if (connected) {
            eventReceived = EVENT_EXCEPTION;
            FlagAnEvent(&eventOccurred);
         }
         else {
            exit(0);
         }
      }
   }
};



/* Class: InstallSession
   purpose: A specialised subclass for doing asynchronous terminal installs */

class InstallSession : public CclSession
{
public:


   // Required for compilation, constructor does nothing but call superclass

   InstallSession(Ccl::Sync protocol) : CclSession(protocol) {}


   /* Reply handler to deal with install terminal replies. Set flag to say
      terminal has been installed. */

   void handleReply(State state, CclScreen* pscr) {
      eventReceived = EVENT_REPLY;
      FlagAnEvent(&eventOccurred);
   }
};


/* Class: ServerSession
   purpose: A specialised subclass for handling Replies from a server */

class ServerSession : public CclSession
{
public:


   // Required for Compilation, constructor does nothing but call superclass

   ServerSession(Ccl::Sync protocol) : CclSession(protocol) {}


   // Handle replies from the server. This is a simple screen generator

   void handleReply(State state, CclScreen* pscr) {

      CclField* pfld;
      int col = 1;
      int row = 1;
      if (state != CclSession::server) {
         cout << "\nScreen From CICS" << endl;
         for (int i=1; i<= pscr->fieldCount(); i++) {
            pfld = pscr->field(i);
            while (pfld->row() > row) {
               cout << endl;
               row++;
               col = 1;
            }
            while (pfld->column() > col) {
               cout << " ";
               col++;
            }
            cout << pfld->text() << " ";
            col = col + pfld->length();
         }
         cout << endl;
         cout << endl;
         FlagAnEvent(&eventOccurred);
      }
   }
};



/* Function: main
   Purpose: Main Program Entry point */

int main(void)
{


   // Define EPI objects

   MyCclEPI* pepi;
   CclTerminal* pterm;
   InstallSession* pInstSession;
   ServerSession* pSvrSession;
   CclField* pfld;

   char input[80];
   unsigned short serverSelected;

	cout << "CICS Transaction Gateway Intermediate EPI C++ Sample" << endl << endl;


   if (CreateAnEvent("eventOccurred",&eventOccurred) == CCLFALSE) {
      cout << "Failed to create event semaphore. Program must end" << endl;
   }


   /* Attempt to create an EPI object with new exception handler. If it fails
      then a throw will occur as it calls the superclass's default handler, not
      the subclassed one. */

   try {
      pepi = new MyCclEPI();
   }
   catch(CclException &ex) {


      // Handle the EPI object creation failure.

      cout << "Unable to create an EPI object" << endl;
      cout << ex.diagnose() << endl;
      cout << "Program cannot continue" << endl;
      return 1;
   }


   // Get a Server from the user

   do {
      cout << "\nSelect a aerver or Q to end\n" << endl;
      for (int i = 1; i <= pepi->serverCount(); i++  ) {
         cout << "    " << i << ". " << pepi->serverName(i) << " - "
              << pepi->serverDesc(i) << endl;
      }

      cout << endl << "Choose server to connect to:";
      cout.flush();
      if (cin.peek()=='\n') cin.ignore(1,'\n');
      cin.getline(input,79,'\n');

      if (input[0] == 'q' || input[0] == 'Q') {
         delete pepi;
         return 0;
      }
      serverSelected = (unsigned short)atoi(input);
   } while (serverSelected < 1 || serverSelected > pepi->serverCount()  );


   /* Create the session objects that will be used in the program, then create
      a SignonCapable terminal with a read timeout of x seconds, no supplied
      user ID or password and specify the default code page. Install this
      terminal asynchronously. */

   pInstSession = new InstallSession(Ccl::async);
   pSvrSession = new ServerSession(Ccl::async);
   pterm = new CclTerminal(pepi->serverName(serverSelected),"","",
                                   CclTerminal::signonCapable,"","",5,0);
   pterm->install(pInstSession,30);
   cout << "Waiting up to 30 Seconds for the terminal to install" << endl;
   DoOtherThings();
   connected = Ccl::yes;


   /* If the terminal fails to install, it will be processed by handle
      exception DoOtherthings will not get flagged and the program will
      terminate. */

   cout << "\nThe following information was set/returned about the terminal :"
        << endl;
   cout << "Server      =" << pterm->serverName() << endl;
   cout << "Devtype     =" << pterm->devType() << endl;
   cout << "Netname     =" << pterm->netName() << endl;
   cout << "Terminal ID =" << pterm->termID() << endl;
   cout << "Read timeout set to '" << pterm->readTimeout() << "' seconds"
        << endl;

   switch (pterm->signonCapability()) {
   case CclTerminal::signonCapable:
      cout << "Terminal is signon capable" << endl;
      break;
   case CclTerminal::signonIncapable:
      cout << "Terminal is signon incapable" << endl;
      break;
   default:
      cout << "Terminal Signon capability is unknown" << endl;
   }

   char txn[80];
   cout << "\nEnter a transaction to run: ";
   cout.flush();

   if (cin.peek()=='\n') cin.ignore(1,'\n');
   cin.getline(txn,79,'\n');


   /* Send the transaction.
      This demonstrates a different way of starting a transaction. Here you
      have full control over the information sent to CICS. */

   CclField* startfld = pterm->screen()->field(1,1);
   startfld->setText(txn);
   pterm->send(pSvrSession);


   // Wait for the server now, but could be doing other things

   CclScreen::AID aid;
   unsigned short resp;

   do {
      cout << "Waiting for a response from the server." << endl;
      DoOtherThings();
      if (eventReceived == EVENT_REPLY) {
         cout << "Terminal is ready for input." << endl;
         do {
            resp = GetResponse(aid);
            if (resp == RESP_BADINPUT) {
               cout << "\nInvalid input, try again.\n" << endl;
            }
         } while (resp == RESP_BADINPUT);
         if (resp == RESP_OK && eventReceived != EVENT_EXCEPTION) {
            pterm->screen()->setAID(aid);
            pterm->send(pSvrSession);
         }
         else {
            resp = RESP_QUIT;
         }
      }
      else {
         resp = RESP_QUIT;
      }
   } while (resp != RESP_QUIT);


   /* Disconnect the terminal, purge it in case a transaction is still running
      due to a possible exception. */

   if (eventReceived == EVENT_EXCEPTION ) {
      cout << "Disconnecting the terminal with a purge." << endl;
      pterm->disconnect(Ccl::yes);
   }
   else {
      cout << "Disconnecting the terminal." << endl;
      pterm->disconnect();


      /* Check to see if an exception occurred trying to disconnect. If it did
         then it could be we tried to disconnect while a conversational
         transaction was running, which would fail so we need to do a purge. */

      if (eventReceived == EVENT_EXCEPTION) {
         cout << "Disconnecting the terminal with a purge due to failure." << endl;
         cout << "Possibly due to a conversational transaction still running." << endl;
         pterm->disconnect(Ccl::yes);
      }
   }


   // Delete all the objects used

   delete pInstSession;
   delete pSvrSession;
   delete pterm;
   delete pepi;

   return 0;
}



// Function: DoOtherThings
// Purpose: To show something else is going on while talking to CICS

void DoOtherThings() {
   int i=0;
   for (i = 1; i<5000;i++) {
      cout << "   Doing something: " << i << " \r";
   }
   cout << "\nWaiting for the server now" << endl;
   WaitOnAnEvent(&eventOccurred);
   ResetAnEvent(&eventOccurred);

}


// Function: GetResponse
// Purpose: Simple routine to allow you to enter the key you want pressed

unsigned short GetResponse(CclScreen::AID &aidkey) {

   cout << "\nChoose from the following responses:" << endl;
   cout << "pf(1-24)" << endl;
   cout << "pa(1-3)" << endl;
   cout << "Enter" << endl;
   cout << "Clear" << endl << endl;
   cout << "q to disconnect" << endl;
   cout << endl<<"Please enter your choice: ";
   cout.flush();

   char input[80];
   if (cin.peek()=='\n') cin.ignore(1,'\n');
   cin.getline(input,79,'\n');


   // Check for q and return a flag to say q had been pressed

   if (input[0] == 'q') {
      return RESP_QUIT;
   }

   if (input[0] == 'p' && input[1] == 'f') {


      // Look for lower case pf, return an error if > 24

      unsigned short x = atoi(&input[2]);
      if (x > 24) {
         return RESP_BADINPUT;
      }
      aidkey = (CclScreen::AID)((int)CclScreen::PF1 + x - 1);
   }
   else if (input[0] == 'p' && input[1] == 'a') {


      // Look for lower case pa, return an error if > 3

      unsigned short x = atoi(&input[2]);
      if (x > 3) {
         return RESP_BADINPUT;
      }
      aidkey = (CclScreen::AID)((int)CclScreen::PA1 + x - 1);

   }


   // Look for enter or clear.

   else if (strcmp(input,"enter") == 0) {
     aidkey = CclScreen::enter;
   }
   else if (strcmp(input,"clear") == 0) {
      aidkey = CclScreen::clear;
   }
   else {
      return RESP_BADINPUT;
   }


   // Good entry return

   return RESP_OK;
}
