/* File Name(s)  : esib1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample shows how to use the CICS TG C++ ESI API
                   by verifying a username and password and then using the
                   supplied credentials to make an ECI call

   Prerequisites:  This sample requires the CICS Transaction EC01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind. IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */



// Include files

#include <string.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include <iostream.h>
#endif

#include <time.h>
#include <stdlib.h>

#include <cicseci.hpp>


// Structures, defines and macro definitions

#define ECI_DATA_LENGTH 18       // Set the length of your commarea here.
#define CHOICE_LENGTH	100      // Set the length of the response for server the choice

#define copyright_notice copyright4cppesib1
char version4cppesib1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

// Main Function

int main(void){
   char chosenServer[8 +1];      // Server name of users choice.
   char progid[4 +1] = "EC01";   // Program to be executed.
   CclBuf commarea("");          // Commarea for use in transactions.
   CclSecAttr *secAttr;          // Security attributes block.
   CclSecTime *secTiming;        // Security time information block.
   tm time;
   char *reply;

   char UserID[9] = "        ";  //Set UserID to blanks.
   char PassWd[9] = "        ";  //Set PassWd to blanks.

   int i, loop;
   char choice[CHOICE_LENGTH];	 //Stores the server choice made by user

   cout << "CICS Transaction Gateway Basic C++ ESI Sample" << endl << endl;


   //Open try block to catch all Class exceptions.

   try {

      //Retrieve list of servers from Client daemon.

      cout << "Servers Available in Client daemon:" << endl;
      CclECI *pEci = CclECI::instance();
      for (i=1;i<=pEci->serverCount();i++) {
         cout << "    " << i << "." << pEci->serverName(i) << endl;
      }


      //Get user choice for server to be used.

      do {


         //Ask user to chose the server to call.

         cout << endl << "Choose Server to connect to:";
         cout.flush();
         if (cin.peek() == '\n') cin.ignore(1, '\n');
         cin.getline(choice, CHOICE_LENGTH, '\n');


         //Check if choice is within range.

         if (atoi(choice) > 0 && atoi(choice) < pEci->serverCount()+1) {
            strcpy(chosenServer,pEci->serverName(atoi(choice)));
            loop=0;
         }
		 else {
			cout << "ERROR: " << "\"" << choice << "\" is not an a valid choice." << endl;
            loop=1;
         }
      }while (loop==1);


      //Get user ID and password for verification by the server.

      cout << "Enter userid for server " << chosenServer << ":";
      cout.flush();
      if (cin.peek() == '\n') cin.ignore(1, '\n');
      cin.getline(UserID, sizeof(UserID), '\n');

      cout << "Enter password for user " << UserID << ":";
      cout.flush();
      if (cin.peek() == '\n') cin.ignore(1, '\n');
      cin.getline(PassWd, sizeof(PassWd), '\n');


      //Open a connection with the chosen server.

      CclConn server(chosenServer, UserID, PassWd);
      CclFlow sflow(Ccl::sync);


      //Verify the userid and password using ESI.

      cout << "Verifying UserID and Password" << endl;
      try {
         secAttr = server.verifyPassword();
      } catch ( CclException& exception) {
         cout << endl << "CclClass ESI exception:" << exception.diagnose()
              << endl;
         return 1;
      }
      cout << "UserID and Password verified." << endl;
      secTiming = secAttr->lastVerifiedTime();
      time = secTiming->get_tm();
      cout << "  Last Verification occurred at: " << asctime(&time) << endl;
      secTiming = secAttr->expiryTime();
      if(secTiming->year() == 0){
         cout << "  Account is non-expiring" << endl;
      } else {
         time = secTiming->get_tm();
         cout << "  Expiry date for account " << UserID << " is:"
              << asctime(&time) << endl;
      }


      //Start a link to the server with the given transaction as input.

      cout << "Calling CICS with transaction " << progid << "...." << endl;
      commarea.setDataLength(ECI_DATA_LENGTH);
      server.link(sflow, progid, &commarea);


      //Check the Execution code and report any failure.

      if (pEci->exCode() != 0) {
         cout << "Call to CICS failed" << endl;
         cout << "Return Code: " << pEci->exCode() << " : "
              << pEci->exCodeText() << endl;
      } else {
         cout << "Program " << progid << " succeeded" << endl;
         //cast reply to a char.
         reply = (char*) commarea.dataArea(0);
         cout << "Reply from CICS: " << reply << endl;
      }
   } catch ( CclException& exception ) {
      //Output exception diagnostics.
      cout << "CclClass exception:" << exception.diagnose() << endl;
   }
   return 0;
}
