      **********************************************************
      *
      * Module Name        EC03.CCP
      *
      * Descriptive Name   CICS External Call Interface
      *
      * Licensed Materials - Property of IBM  
      *  
      * 5724-I81,5725-B65,5655-Y20 
      *  
      * (C) Copyright IBM Corp. 2007, 2014 All Rights Reserved.  
      *  
      * US Government Users Restricted Rights - Use, duplication or  
      * disclosure restricted by GSA ADP Schedule Contract with  
      * IBM Corp.  
      *
      * Status: Version 9 Release 1 
      *
      * EC03 is a Basic CICS Server Cobol sample program which is used
      * with the frontend sample programs to demonstrate the use of
      * External Call Interface (ECI) with channels and containers.
      *
      * Notes :-
      * This server program demonstrates the use of channels and
      * containers in a CICS program. It queries the contents of
      * container INPUTDATA and places the length into container
      * INPUTDATALENGTH. The current date and time on the CICS server
      * are returned in container CICSDATETIME. The input data or a
      * failure message is returned in container OUTPUTMESSAGE. If
      * no channel is passed to the program it abends with abend
      * code NOCH. If a CICS PUT CONTAINER command fails the program
      * abends with abend code PUTC.
      *
      * On CICS Transaction Server the CCSID can be placed into
      * container INPUTDATACCSID. To enable this, uncomment the
      * two EXEC CICS calls marked "Only supported on CICS TS".
      *
      * Note that channel and container names are case sensitive.
      *
      * The following code is sample code created by IBM Corporation.
      * This sample code is not part of any standard IBM product and
      * is provided to you solely for the purpose of assisting you in
      * the development of your applications.  The code is provided
      * 'as is', without warranty or condition of any kind.  IBM shall
      * not be liable for any damages arising out of your use of the
      * sample code, even if IBM has been advised of the possibility
      * of such damages.
      *

       IDENTIFICATION DIVISION.
       PROGRAM-ID. EC03.

       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.

      *  Container names
      *  Channel and container names are case sensitive
       01 DATETIME-CONT      PIC X(16) VALUE 'CICSDATETIME'.
       01 INPUT-CONT         PIC X(16) VALUE 'INPUTDATA'.
       01 OUTPUT-CONT        PIC X(16) VALUE 'OUTPUTMESSAGE'.
       01 LENGTH-CONT        PIC X(16) VALUE 'INPUTDATALENGTH'.
       01 CCSID-CONT         PIC X(16) VALUE 'INPUTDATACCSID'.

      *  Data fields used by the program
       01 INPUTLENGTH        PIC S9(8) COMP-4.
       01 CURRENTTIME        PIC S9(15) COMP-3.
       01 CHANNELNAME        PIC X(16) VALUE SPACES.
       01 INPUTSTRING        PIC X(72) VALUE SPACES.
       01 OUTPUTSTRING       PIC X(88) VALUE SPACES.
       01 DATE-TIME.
         03 DATESTRING         PIC X(10) VALUE SPACES.
         03 DATETIMESEP        PIC X(1) VALUE SPACES.
         03 TIMESTRING         PIC X(8) VALUE SPACES.
       01 INPUTCCSID         PIC S9(8) COMP-4 VALUE 0.
       01 INPUTCCSID-DISP    PIC Z(8) DISPLAY VALUE SPACES.
       01 RESPCODE           PIC S9(8) COMP-4 VALUE 0.
       01 RESPCODE2          PIC S9(8) COMP-4 VALUE 0.
       01 DATALENGTH         PIC S9(8) COMP-4.

       PROCEDURE DIVISION.
       MAIN-PROCESSING SECTION.

      *  Get name of channel
           EXEC CICS ASSIGN CHANNEL(CHANNELNAME)
                            END-EXEC.

      *  If no channel passed in, terminate with abend code NOCH
           IF CHANNELNAME = SPACES THEN
               EXEC CICS ABEND ABCODE('NOCH') NODUMP
                            END-EXEC
           END-IF.

      *  Only supported on CICS TS
      *
      *  Get CCSID of container INPUTDATA
      *    EXEC CICS GET CONTAINER(INPUT-CONT)
      *                     CHANNEL(CHANNELNAME)
      *                     CCSID(INPUTCCSID)
      *                     CONVERTST(DFHVALUE(NOCONVERT))
      *                     NODATA
      *                     FLENGTH(DATALENGTH)
      *                     RESP(RESPCODE)
      *                     RESP2(RESPCODE2)
      *                     END-EXEC.
      *    MOVE INPUTCCSID TO INPUTCCSID-DISP.

      *  Read contents and length of container INPUTDATA
           MOVE LENGTH OF INPUTSTRING TO INPUTLENGTH.
           EXEC CICS GET CONTAINER(INPUT-CONT)
                            CHANNEL(CHANNELNAME)
                            FLENGTH(INPUTLENGTH)
                            INTO(INPUTSTRING)
                            RESP(RESPCODE)
                            RESP2(RESPCODE2)
                            END-EXEC.

      *  Check response code
           EVALUATE RESPCODE

      *  Container not passed in
             WHEN DFHRESP(CONTAINERERR)
               STRING INPUT-CONT
                            DELIMITED BY SPACE
                      ' container was not passed to the program'
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING

      *  Invalid data length
             WHEN DFHRESP(LENGERR)
               STRING INPUT-CONT
                            DELIMITED BY SPACE
                      ' invalid input data length'
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING

      *  Container is BIT not CHAR
             WHEN DFHRESP(CCSIDERR)
               IF RESPCODE2 = 3
                 STRING 'Container '
                            DELIMITED BY SIZE
                        INPUT-CONT
                            DELIMITED BY SPACE
                        ' type is BIT, not CHAR'
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING
               ELSE
                 STRING 'The GET CONTAINER command returned an',
                        ' unexpected CCSIDERR condition'
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING
               END-IF

      *  Read from container OK
             WHEN DFHRESP(NORMAL)
               STRING 'Input data was: '
                            DELIMITED BY SIZE
                      INPUTSTRING
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING

               EXEC CICS PUT CONTAINER(LENGTH-CONT)
                            FROM(INPUTLENGTH)
                            FLENGTH(LENGTH OF INPUTLENGTH)
                            BIT RESP(RESPCODE) END-EXEC

               IF RESPCODE NOT = DFHRESP(NORMAL)
                 EXEC CICS ABEND ABCODE('PUTC') NODUMP
                            END-EXEC
               END-IF

      *  Other response code
             WHEN OTHER
               STRING 'The GET CONTAINER command returned an',
                      ' unexpected response code'
                            DELIMITED BY SIZE
                            INTO OUTPUTSTRING END-STRING

           END-EVALUATE.

      *  Place output string in container OUTPUTMESSAGE
           EXEC CICS PUT CONTAINER(OUTPUT-CONT)
                            FROM(OUTPUTSTRING)
                            FLENGTH(LENGTH OF OUTPUTSTRING)
                            CHAR
                            RESP(RESPCODE)
                            END-EXEC.

      *  Check return code
           IF RESPCODE NOT = DFHRESP(NORMAL)
               EXEC CICS ABEND ABCODE('PUTC') NODUMP
                            END-EXEC
           END-IF.

      *  Get the current time and date
           EXEC CICS ASKTIME ABSTIME(CURRENTTIME)
                            END-EXEC.

      *  Format date and time
           EXEC CICS FORMATTIME ABSTIME(CURRENTTIME)
                            DDMMYYYY(DATESTRING) DATESEP('/')
                            TIME(TIMESTRING) TIMESEP(':')
                            RESP(RESPCODE)
                            END-EXEC.

      *  Check return code
           IF RESPCODE NOT = DFHRESP(NORMAL)
               STRING 'Failed' DELIMITED BY SIZE
                            INTO DATESTRING END-STRING
           END-IF.

      *  Place date and time into container CICSDATETIME
           EXEC CICS PUT CONTAINER(DATETIME-CONT)
                            FROM(DATE-TIME)
                            FLENGTH(LENGTH OF DATE-TIME)
                            CHAR
                            RESP(RESPCODE)
                            END-EXEC.

      *  Check return code
           IF RESPCODE NOT = DFHRESP(NORMAL)
               EXEC CICS ABEND ABCODE('PUTC') NODUMP
                            END-EXEC
           END-IF.

      *  Only supported on CICS TS
      *
      *  Place CCSID into container INPUTDATACCSID
      *    EXEC CICS PUT CONTAINER(CCSID-CONT)
      *                     FROM(INPUTCCSID-DISP)
      *                     FLENGTH(LENGTH OF INPUTCCSID-DISP)
      *                     CHAR
      *                     RESP(RESPCODE)
      *                     END-EXEC.

      *  Check return code
      *    IF RESPCODE NOT = DFHRESP(NORMAL)
      *        EXEC CICS ABEND ABCODE('PUTC') NODUMP
      *                     END-EXEC
      *    END-IF.

      *  Finish
           EXEC CICS RETURN END-EXEC.

           EXIT.
