/* Module Name       epix1.c

   Descriptive Name  EPI Exit Library Sample

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 1 

         The following code is sample code created by IBM
         Corporation.  This sample code is not part of any standard
         IBM product and is provided to you solely for the purpose
         of assisting you in the development of your applications.
         The code is provided 'AS IS', without warranty or
         condition of any kind. IBM shall not be liable for any
         damages arising out of your use of the sample code, even
         if IBM has been advised of the possibility of such damages.



   NOTES :-

      This is a sample EPI exits object. It performs no function
      but does provide a skeleton containing all the exit functions
      with details of valid return codes and their meanings.

      When writing exits, the following rules must be obeyed:
      1. No EPI or ECI calls must be made from the exit
      2. Exits should contain the minimum possible executable code
         (to minimise impact on performance)
      3. All exit code must be re-entrant and thread safe.
      4. The primary entry point to this module must be named
         CICS_EpiExitInit.  The names of the actual exits may
         be changed but the parameter lists must not be changed.
      5. An EPI user exit program should contain valid entry points
         for all of the user exit functions. */


/* HEADER FILES */

#include <cics_epi.h>
#include <cicsepix.h>
#include <string.h>
#include <ctype.h>

#define copyright_notice copyright4cepix1
char version4cepix1[] =
"\nCICS Transaction Gateway"
" Version 9.1.0.4"
" Build Level c910-20231012-1355" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

/* Main entry point of dynamically loaded object.
   This function returns a structure of addresses of exit functions.
   Note that ALL exits must be defined if any are defined.
   If any exit address is set to NULL no exits will be called. */

#if defined(CICS_W32)
  #define CICSEXPORT __declspec ( dllexport )
#else
  #define CICSEXPORT
#endif

void CICSEXPORT CICSEXIT CICS_EPIEXITINIT(CICS_EpiExitList_t *ExitList)
{
    ExitList->InitializeExit        = (CICS_EpiInitializeExitFn CICSPTR *)&CICS_EpiInitializeExit;
    ExitList->TerminateExit         = (CICS_EpiTerminateExitFn CICSPTR *)&CICS_EpiTerminateExit;
    ExitList->AddTerminalExit       = (CICS_EpiAddTerminalExitFn CICSPTR *)&CICS_EpiAddTerminalExit;
    ExitList->StartTranExtendedExit = (CICS_EpiStartTranExtendedExitFn CICSPTR *)&CICS_EpiStartTranExtendedExit;
    ExitList->ReplyExit             = (CICS_EpiReplyExitFn CICSPTR *)&CICS_EpiReplyExit;
    ExitList->DelTerminalExit       = (CICS_EpiDelTerminalExitFn CICSPTR *)&CICS_EpiDelTerminalExit;
    ExitList->GetEventExit          = (CICS_EpiGetEventExitFn CICSPTR *)&CICS_EpiGetEventExit;
    ExitList->TranFailedExit        = (CICS_EpiTranFailedExitFn CICSPTR *)&CICS_EpiTranFailedExit;
    ExitList->SystemIdExit          = (CICS_EpiSystemIdExitFn CICSPTR *)&CICS_EpiSystemIdExit;
    ExitList->TermIdInfoExit        = (CICS_EpiTermIdInfoExitFn CICSPTR *)&CICS_EpiTermIdInfoExit;
}


/* InitializeExit:
   Called from CICS_EpiInitialize after successful initialisation, but
   prior to returning to the caller

   Permitted return values:
     CICS_EXIT_OK               No problems - call exits where appropriate
     CICS_EXIT_NO_EXITS         No problems - do not call subsequent exits
                                    (used by shipped dummy library)
     CICS_EXIT_CANT_INIT_EXITS  Exits cannot be used due to an
                                    (unspecified) error condition
     CICS_EXIT_USER_BASE + N    User specified value

   Notes:
     Any value except CICS_EXIT_OK will inhibit the calling of subsequent exits */

cics_sshort_t CICSEXIT CICS_EpiInitializeExit( cics_ulong_t      Version,
                                               CICS_EpiAnchor_t *Anchor)

{
        *Anchor = (void *)(-1);

        return  CICS_EXIT_OK;
}


/* TerminateExit:
   Called from CICS_EpiTerminate immediately prior to the destruction of the
   EPI data structures.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_STORAGE      Exit cannot tidy up its storage
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiTerminateExit(CICS_EpiAnchor_t Anchor)
{
        return CICS_EXIT_OK;
}


/* AddTerminalExit:
   Called from CICS_EpiAddTerminal after validation of the input parameters
   but before the add of the terminal.  This exit may change any of the input
   parameters NameSpace, System, NetName, DevType.

   Permitted return values:
     CICS_EXIT_OK                No problems
     CICS_EXIT_DONT_ADD_TERMINAL Do not add this terminal
     CICS_EXIT_BAD_ANCHOR        Invalid anchor value supplied
     CICS_EXIT_BAD_PARM          One or more parameters bad
     CICS_EXIT_USER_BASE + N     User specified value

   Notes:
     After a return of CICS_EXIT_DONT_ADD_TERMINAL, CICS_EpiAddTerminal will
     return CICS_EPI_ERR_FAILED, and a subsequent call to CICS_EpiGetSysError
     will return a cause of 5.
     This exit must take great care when changing the system specified if
     the system is not blank, since in this case the calling application
     will not expect a change and may behave badly as a result. */

cics_sshort_t CICSEXIT CICS_EpiAddTerminalExit( CICS_EpiAnchor_t Anchor,
                                                cics_char_t      NameSpace[1025],
                                                cics_char_t      System[CICS_EPI_SYSTEM_MAX+1],
                                                cics_char_t      NetName[CICS_EPI_NETNAME_MAX+1],
                                                cics_char_t      DevType[CICS_EPI_DEVTYPE_MAX+1])
{
        return CICS_EXIT_OK;
}


/* StartTranExtendedExit:
   Called from CICS_EpiStartTran after validation of input parameters.
   This exit must not change any of the input parameters

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT
CICS_EpiStartTranExtendedExit( CICS_EpiAnchor_t Anchor,
                               cics_ushort_t    TermIndex,
                               cics_char_t      TransId[CICS_EPI_TRANSID_MAX+1],
                               cics_ubyte_t    *Data,
                               cics_ushort_t    Size)
{
        return CICS_EXIT_OK;
}


/* ReplyExit:
   Called from CICS_EpiReply after validation of input parameters.
   This exit must not change the input parameters.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiReplyExit( CICS_EpiAnchor_t Anchor,
                                          cics_ushort_t    TermIndex,
                                          cics_ubyte_t    *Data,
                                          cics_ushort_t    Size)
{
        return CICS_EXIT_OK;
}


/* DelTerminalExit:
   Called from CICS_EpiDelTerminal after validation of input parameters.
   This exit must not change the input parameters.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiDelTerminalExit( CICS_EpiAnchor_t Anchor,
                                                cics_ushort_t    TermIndex)
{
        return CICS_EXIT_OK;
}


/* GetEventExit:
   Called from CICS_EpiGetEvent after validation of input parameters.
   This exit must not change the input parameters.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiGetEventExit( CICS_EpiAnchor_t       Anchor,
                                             cics_ushort_t          TermIndex,
                                             CICS_EpiWait_t         Wait,
                                             CICS_EpiEventData_t   *Event)
{
        return CICS_EXIT_OK;
}


/* TranFailedExit:
   Called on receipt of an END_TRAN or END_TERM event with an associated abend.
   This exit must not change the input parameters.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiTranFailedExit( CICS_EpiAnchor_t       Anchor,
                                               cics_ushort_t          TermIndex,
                                               CICS_EpiWait_t         Wait,
                                               CICS_EpiEventData_t   *Event)
{
        return CICS_EXIT_OK;
}


/* SystemIdExit:
   Called from CICS_EpiAddTerminal after a failure to add a terminal.
   This exit may change the input parameters
   NameSpace, System, NetName, DevType
   This exit will be called repeatedly while it returns CICS_EXIT_OK
   until it returns a system which can successfully fulfill the request.
   A return code of CICS_EXIT_DONT_ADD_TERMINAL will force an exit
   from the retry loop.

   Permitted return values:
     CICS_EXIT_OK                No problems
     CICS_EXIT_BAD_ANCHOR        Invalid anchor value supplied
     CICS_EXIT_BAD_PARM          One or more parameters bad
     CICS_EXIT_DONT_ADD_TERMINAL Do not make any further retry attempts
     CICS_EXIT_USER_BASE + N     User specified value

   NOTE:
     UserID and PassWord are included for compatibility with other
     platforms but are not used on UNIX and Linux platforms */

cics_sshort_t CICSEXIT CICS_EpiSystemIdExit( CICS_EpiAnchor_t Anchor,
                                             cics_char_t      NameSpace[1025],
                                             cics_char_t      System[CICS_EPI_SYSTEM_MAX+1],
                                             cics_char_t      NetName[CICS_EPI_NETNAME_MAX+1],
                                             cics_char_t      DevType[CICS_EPI_DEVTYPE_MAX+1],
                                             cics_char_t      FailedSystem[CICS_EPI_SYSTEM_MAX+1],
                                             cics_sshort_t    Reason,
                                             cics_ulong_t     SubReason,
                                             cics_char_t      UserId[64],
                                             cics_char_t      PassWord[64])
{

       /*
        * If the workload manager is active ( Windows only ), and user exits
        * are being stacked with workload manager exits, this exit will only
        * be called if the workload manager returns CICS_EXIT_OK.
        */
        return CICS_EXIT_DONT_ADD_TERMINAL;
}


/* TermIdInfoExit:
   Called from CICS_EpiAddTerminal after successful installation of terminal
   This exit must not change the input parameters.

   Permitted return values:
     CICS_EXIT_OK               No problems
     CICS_EXIT_BAD_ANCHOR       Invalid anchor value supplied
     CICS_EXIT_BAD_PARM         One or more parameters bad
     CICS_EXIT_USER_BASE + N    User specified value */

cics_sshort_t CICSEXIT CICS_EpiTermIdInfoExit( CICS_EpiAnchor_t            Anchor,
                                               cics_ulong_t                Version,
                                               cics_ushort_t              *TermIndex,
                                               CICS_EpiDetails_t          *Details)
{
        return CICS_EXIT_OK;
}
